@@ -15,6 +15,7 @@
 #include "xb-builder-source-ctx-private.h"
 #include "xb-builder-source-private.h"
 #include "xb-lzma-decompressor.h"
+#include "xb-zstd-decompressor.h"
 
 typedef struct {
 	GInputStream *istream;
@@ -527,6 +528,18 @@ xb_builder_source_load_lzma_cb(XbBuilderSource *self,
 	return g_converter_input_stream_new(istream, conv);
 }
 
+static GInputStream *
+xb_builder_source_load_zstd_cb(XbBuilderSource *self,
+			       XbBuilderSourceCtx *ctx,
+			       gpointer user_data,
+			       GCancellable *cancellable,
+			       GError **error)
+{
+	GInputStream *istream = xb_builder_source_ctx_get_stream(ctx);
+	g_autoptr(GConverter) conv = G_CONVERTER(xb_zstd_decompressor_new());
+	return g_converter_input_stream_new(istream, conv);
+}
+
 static void
 xb_builder_source_adapter_free(XbBuilderSourceAdapter *item)
 {
@@ -581,6 +594,11 @@ xb_builder_source_init(XbBuilderSource *self)
 				      xb_builder_source_load_lzma_cb,
 				      NULL,
 				      NULL);
+	xb_builder_source_add_adapter(self,
+				      "application/zstd",
+				      xb_builder_source_load_zstd_cb,
+				      NULL,
+				      NULL);
 }
 
 /**
