@@ -23,11 +23,12 @@ struct _XbOpcode {
 	guint8 tokens_len;
 	const gchar *tokens[XB_OPCODE_TOKEN_MAX + 1];
 	GDestroyNotify destroy_func;
+	guint8 level;
 };
 
 #define XB_OPCODE_INIT()                                                                           \
 	{                                                                                          \
-		0, 0, NULL, 0, {NULL}, NULL                                                        \
+		0, 0, NULL, 0, {NULL}, NULL, 0                                                     \
 	}
 
 /**
@@ -49,15 +50,15 @@ xb_opcode_steal(XbOpcode *op_ptr)
 }
 
 void
-xb_opcode_init(XbOpcode *opcode,
+xb_opcode_init(XbOpcode *self,
 	       XbOpcodeKind kind,
 	       const gchar *str,
 	       guint32 val,
 	       GDestroyNotify destroy_func);
 void
-xb_opcode_clear(XbOpcode *opcode);
+xb_opcode_clear(XbOpcode *self);
 void
-xb_opcode_bind_init(XbOpcode *opcode);
+xb_opcode_bind_init(XbOpcode *self);
 gboolean
 xb_opcode_is_binding(XbOpcode *self);
 G_DEPRECATED_FOR(xb_value_bindings_bind_str)
@@ -77,12 +78,17 @@ xb_opcode_get_tokens(XbOpcode *self);
 gchar *
 xb_opcode_get_sig(XbOpcode *self);
 void
-xb_opcode_bool_init(XbOpcode *opcode, gboolean val);
+xb_opcode_bool_init(XbOpcode *self, gboolean val);
 gboolean
 xb_opcode_has_flag(XbOpcode *self, XbOpcodeFlags flag);
 void
 xb_opcode_add_flag(XbOpcode *self, XbOpcodeFlags flag);
 
+void
+xb_opcode_set_level(XbOpcode *self, guint8 level);
+guint8
+xb_opcode_get_level(XbOpcode *self);
+
 G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC(XbOpcode, xb_opcode_clear)
 
 G_END_DECLS
