@@ -224,9 +224,9 @@ xb_predicate_func(void)
 		     {"@a=='b'", "'a',attr(),'b',eq()"},
 		     {"'a'<'b'", "'a','b',lt()"},
 		     {"999>=123", "999,123,ge()"},
-		     {"not(0)", "0,not()"},
+		     {"not(0)", "0^1,not()"},
 		     {"@a", "'a',attr(),'(null)',ne()"},
-		     {"not(@a)", "'a',attr(),not()"},
+		     {"not(@a)", "'a'^1,attr()^1,not()"},
 		     {"'a'=", "'a',eq()"},
 		     {"='b'", "'b',eq()"},
 		     {"999=\'b\'", "999,'b',eq()"},
@@ -235,9 +235,10 @@ xb_predicate_func(void)
 		     {"text()~='beef'", "text(),'beef'[beef],search()"},
 		     {"@type~='dead'", "'type',attr(),'dead',search()"},
 		     {"2", "2,position(),eq()"},
-		     {"text()=lower-case('firefox')", "text(),'firefox',lower-case(),eq()"},
+		     {"text()=lower-case('firefox')", "text(),'firefox'^1,lower-case(),eq()"},
 		     {"$'a'=$'b'", "$'a',$'b',eq()"},
-		     {"('a'='b')&&('c'='d')", "'a','b',eq(),'c','d',eq(),and()"},
+		     {"('a'='b')&&('c'='d')", "'a'^1,'b'^1,eq()^1,'c'^1,'d'^1,eq()^1,and()"},
+		     {"text()==('a','b','c')", "text(),'c'^1,'b'^1,'a'^1,in()"},
 		     /* sentinel */
 		     {NULL, NULL}};
 	const gchar *invalid[] = {"text(",
@@ -291,6 +292,7 @@ xb_predicate_optimize_func(void)
 		     {"lower-case('Fire')", "'fire'"},
 		     {"upper-case('Τάχιστη')", "'ΤΆΧΙΣΤΗ'"},
 		     {"upper-case(lower-case('Fire'))", "'FIRE'"}, /* 2nd pass */
+		     {"text()==('a','b','c')", "text(),'c'^1,'b'^1,'a'^1,in()"},
 		     /* sentinel */
 		     {NULL, NULL}};
 	const gchar *invalid[] = {"'a'='b'", "123>=999", "not(1)", NULL};
@@ -487,6 +489,40 @@ xb_builder_source_lzma_func(void)
 	g_assert_nonnull(silo);
 }
 
+static void
+xb_builder_source_zstd_func(void)
+{
+	gboolean ret;
+	g_autofree gchar *path = NULL;
+	g_autofree gchar *tmp_xmlb = g_build_filename(g_get_tmp_dir(), "temp.xmlb", NULL);
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GFile) file = NULL;
+	g_autoptr(GFile) file_src = NULL;
+	g_autoptr(XbBuilder) builder = xb_builder_new();
+	g_autoptr(XbBuilderSource) source = xb_builder_source_new();
+	g_autoptr(XbSilo) silo = NULL;
+
+	/* import a source file */
+	path = g_test_build_filename(G_TEST_DIST, "test.xml.zstd", NULL);
+	file_src = g_file_new_for_path(path);
+	if (!g_file_query_exists(file_src, NULL)) {
+		g_test_skip("does not work in subproject test");
+		return;
+	}
+	ret = xb_builder_source_load_file(source,
+					  file_src,
+					  XB_BUILDER_SOURCE_FLAG_NONE,
+					  NULL,
+					  &error);
+	g_assert_no_error(error);
+	g_assert_true(ret);
+	xb_builder_import_source(builder, source);
+	file = g_file_new_for_path(tmp_xmlb);
+	silo = xb_builder_ensure(builder, file, XB_BUILDER_COMPILE_FLAG_NONE, NULL, &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(silo);
+}
+
 static void
 xb_builder_chained_adapters_func(void)
 {
@@ -1364,6 +1400,25 @@ xb_xpath_func(void)
 	g_assert_nonnull(str);
 	g_debug("\n%s", str);
 
+	/* query with predicate 'in' */
+	xb_silo_set_profile_flags(silo, XB_SILO_PROFILE_FLAG_OPTIMIZER);
+	n = xb_silo_query_first(
+	    silo,
+	    "components/component/id[text()=('gimp.desktop','another.desktop','dave.desktop')]",
+	    &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(n);
+	g_assert_cmpstr(xb_node_get_text(n), ==, "gimp.desktop");
+	g_clear_object(&n);
+
+	/* query with predicate 'in' -- single-entry tuple */
+	xb_silo_set_profile_flags(silo, XB_SILO_PROFILE_FLAG_OPTIMIZER);
+	n = xb_silo_query_first(silo, "components/component/id[text()=('gimp.desktop')]", &error);
+	g_assert_no_error(error);
+	g_assert_nonnull(n);
+	g_assert_cmpstr(xb_node_get_text(n), ==, "gimp.desktop");
+	g_clear_object(&n);
+
 	/* query with predicate logical and */
 	n = xb_silo_query_first(
 	    silo,
@@ -2744,6 +2799,7 @@ main(int argc, char **argv)
 	g_test_add_func("/libxmlb/builder{custom-mime}", xb_builder_custom_mime_func);
 	g_test_add_func("/libxmlb/builder{chained-adapters}", xb_builder_chained_adapters_func);
 	g_test_add_func("/libxmlb/builder{source-lzma}", xb_builder_source_lzma_func);
+	g_test_add_func("/libxmlb/builder{source-zstd}", xb_builder_source_zstd_func);
 	g_test_add_func("/libxmlb/builder-node", xb_builder_node_func);
 	g_test_add_func("/libxmlb/builder-node{token-max}", xb_builder_node_token_max_func);
 	g_test_add_func("/libxmlb/builder-node{info}", xb_builder_node_info_func);
